// Copyright 1999-2018. Plesk International GmbH. All rights reserved.

/**
 * Header Indicator class
 *
 * @constructor
 */
var Indicator = function () {
    this.baseUrl = null;

    this.pollTimeout = 60000;

    this.pollId = null;

    this.percentRank = null;

    this.scoreRank = {
        current: 0,
        total: 0,
    };

    this.isFetching = false;

    this.elements = '.main-header-navbar-nav__item--advisor-1, .r-main-header-navbar-nav__item--advisor-1';

    this.init = function (baseUrl) {
        this.baseUrl = baseUrl;

        this.loadRating();

        Jsw.Observer.append(function (event) {
            this.updateRank(event.ratings);
            this.updateElements();
        }.bind(this), 'update-header-indicator');

        return this;
    };

    /**
     * Update indicator ui elements
     */
    this.updateElements = function() {
        $$(this.elements).each(function(item) {
            this.updateItemTooltip(item);
            this.updateItemBadge(item);
        }.bind(this));
    };

    /**
     * Update item badge
     *
     * @param item
     * @returns {*}
     */
    this.updateItemBadge = function (item) {
        var badge = null;

        var selector = 'span.label';
        var badgeElem = item.select(selector);

        var labelType = this.getLabelType();

        if (!labelType && badgeElem.length) {
            badgeElem.first().remove();
        } else if (labelType && !badgeElem.length) {
            Jsw.render(
                item.select('a').length ? item.select('a').first() : item,
                Jsw.createElement(selector, '!')
            );

            badge = item.select(selector).first();
        } else if (badgeElem.length) {
            badge = badgeElem.first();
        }

        if (badge) {
            badge.removeClassName(badge.className).addClassName('label label-' + labelType);
        }

        return badge;
    };

    /**
     * Return badge type depend on percentRank
     *
     * @returns {*}
     */
    this.getLabelType = function () {
        if (!this.percentRank) {
            return null
        }

        if (this.percentRank <= Indicator.MEDIUM_RATING) {
            return 'danger';
        } else if (this.percentRank > Indicator.MEDIUM_RATING && this.percentRank <= Indicator.HIGH_RATING) {
            return 'warning';
        }

        return null;
    };

    /**
     * Start poll
     *
     * @returns {number}
     */
    this.poll = function () {
        this.clearPoll();

        this.pollId = setTimeout(function () {
            this.loadRating().then(function () {
                this.poll();
            }.bind(this)).catch(function () {
                this.poll();
            }.bind(this));
        }.bind(this), this.pollTimeout);

        return this.pollId;
    };

    /**
     * Clear poll
     */
    this.clearPoll = function () {
        if (this.pollId) {
            clearTimeout(this.pollId);
            this.pollId = null;
        }
    };

    /**
     * Load server rating
     *
     * @returns {Promise}
     */
    this.loadRating = function (recalculate) {
        var self = this;

        this.isFetching = true;
        return new Promise(function (resolve, reject) {
            new Ajax.Request(self.baseUrl + '/rating/all', {
                method: 'get',
                parameters: {
                    ignoreContext: true,
                    initStatistics: true,
                    recalculate: recalculate,
                },
                onSuccess: function (response) {
                    var data = response.responseJSON;

                    if (data.status === 'error') {
                        return reject('Unknown error');
                    }

                    self.updateRank(data);
                    self.updateElements();

                    self.isFetching = false;
                    resolve(data);
                },
                onFailure: function (reason) {
                    self.isFetching = false;
                    reject(reason.responseJSON);
                },
                onException: function (transport, exception) {
                    self.isFetching = false;
                    reject(exception);
                },
                on400: function (reason) {
                    self.isFetching = false;
                    reject(reason.responseJSON);
                },
                on500: function (reason) {
                    self.isFetching = false;
                    reject(reason.responseJSON);
                },
            });
        });
    };

    /**
     * Update current rank values
     *
     * @param ratings List of statues
     */
    this.updateRank = function (ratings) {
        var items = Object.values(ratings);

        var current = 0;
        var total = 0;
        items.forEach(function (item) {
            current += item.current;
            total += item.total;
        });

        this.scoreRank = {
            total: Math.round(total),
            current: Math.round(current),
        };

        this.percentRank = total ? Math.round(100 * current / total) : 0;
    };

    /**
     * Update item tooltip
     *
     * @param item
     */
    this.updateItemTooltip = function (item) {
        var description = 'Your current rating is ' + this.scoreRank.current +' of ' + this.scoreRank.total;

        item.setAttribute('data-description', description);
        Jsw.Tooltip.init(item, {
            text: item.getAttribute('data-description'),
        });
    };
};

Indicator.MEDIUM_RATING = 50;
Indicator.HIGH_RATING = 80;

PleskExt.Advisor.indicator = new Indicator();
